#version 330
#extension GL_ARB_explicit_uniform_location : enable

#define SKIP_RAD 0.2
#define DENSITY_TAU_VIEW_SPACE 0.005

layout( location = 0 ) uniform mat4 viewMat;
layout( location = 4 ) uniform mat4 projMat;
layout( location = 8 ) uniform float maxLifeTime;
layout( location = 10 )uniform sampler3D fireTex;
layout( location = 11 ) uniform sampler2D framebufferDepthTexture;
layout( location = 12 ) uniform vec2 screenResolution;
layout( location = 13 ) uniform float frontClipPlaneDistance;
layout( location = 14 ) uniform mat4 invProjMatrix;
layout( location = 100 ) uniform float glowmod;

in vec2  texCoord;
flat in float fLifetime;
flat in float fType;

flat in float particleRadius;
flat in vec4  particleCenterPosition; //->Particle Position in Camera Space
in vec4       billboardPoint; //-> in Cam space



layout(location = 0, index = 0) out vec4 fragColor;




float getDepthViewSpace() //actually just unprojecting
{
	vec2 xy =  gl_FragCoord.xy / screenResolution;
	float z = texture( framebufferDepthTexture, xy );
	z = z * 2.0 - 1.0; //NDC
	xy = xy * 2.0 - 1.0;
	vec4 projectedPos = vec4( xy, z, 1.0 );
	projectedPos = invProjMatrix * projectedPos;
	projectedPos/=projectedPos.w; 
	return projectedPos.z;
}



float opacityViewSpace()
{
	float r = particleRadius;
	vec3 Q = billboardPoint.xyz / billboardPoint.w;
	vec3 P = particleCenterPosition.xyz / particleCenterPosition.w;
	float tau = DENSITY_TAU_VIEW_SPACE;


	float alpha = 0;

	float d = length( P.xy - Q.xy );
	if( d < r ){
		float w = sqrt( r*r - d*d ); //sphere

		float OccluderDepth = getDepthViewSpace(); //along negative x axis...so this is negative

		float Front = max( OccluderDepth, P.z + w );  //particle center - sphere radius.. entrance point
		Front = min( Front, -frontClipPlaneDistance );

		float Back = max( OccluderDepth, P.z - w ); //sample through the spherical billboard until either back or depth occluder
		
		float ds =  Front - Back; //travel distance in the particle sphere
		alpha = 1.0 - exp( -tau * (1-d/r) * ds );
	}

	return alpha;
}




void main(void)
{
	

	float rad = dot( texCoord-0.5, texCoord-0.5 );
	if( rad > SKIP_RAD )  discard;

	/*fragColor = vec4( 1.0 - smoothstep( 0.0, SKIP_RAD, rad  ) );
	
	float lifePerc = fLifetime / maxLifeTime;
	
	//fragColor = texture( fireTex, vec3(texCoord, lifePerc ) );
	
	fragColor.a *= (1.0 - smoothstep( 0.0, SKIP_RAD, rad ));
	fragColor.a *= smoothstep( 0.0, maxLifeTime * 0.25, fLifetime );
	//fragColor.a *= (1.0 - smoothstep( maxLifeTime * 0.35 , maxLifeTime, fLifetime ));
	fragColor.a *= (1.0 - lifePerc);
	
	fragColor.r *= lifePerc;*/



	
	/*fragColor.r = smoothstep( 0.0, maxLifeTime * 0.2, fLifetime ) * fragColor.r;
	fragColor.g = (1.0 - smoothstep( 0.0, maxLifeTime, fLifetime )) * fragColor.g;
	fragColor.g -= (smoothstep ( maxLifeTime * 0.3, maxLifeTime, fLifetime )) * fragColor.g;
	fragColor.b = (1.0 - smoothstep( 0.0, maxLifeTime * 0.15, fLifetime )) * fragColor.b;	*/


	//from sparks:
	float outside = 1.0 - smoothstep( 0.0, 0.2, rad );
	
	fragColor = vec4( 1.0, 1.0, 1.0, outside * 0.75 );
	
	fragColor.a = (1.0 - smoothstep( maxLifeTime * 0.75, maxLifeTime, fLifetime )) * fragColor.a;
	
	fragColor.r = (smoothstep( maxLifeTime * 0.15, maxLifeTime * 0.75, fLifetime )) * fragColor.r;
	fragColor.g = (smoothstep( maxLifeTime * 0.25, maxLifeTime, fLifetime )) * fragColor.g;
	fragColor.g = max( fragColor.g - smoothstep( maxLifeTime * 0.35, maxLifeTime, fLifetime ) * fragColor.g, 0.0);
	fragColor.b = (1.0 - smoothstep( maxLifeTime * 0.0, maxLifeTime * 0.55, fLifetime )) * fragColor.b;
	//end


	fragColor.a = opacityViewSpace() * outside;
	if( fragColor.a == 0.0 ) discard;

	fragColor.rgb *= glowmod;
	fragColor.rgb *= fragColor.a;

}

