#version 330
#extension GL_ARB_explicit_uniform_location : enable

#define QUADHEIGHT 0.75
#define QUADWIDTH  0.75

layout( location = 0 ) uniform mat4 viewMat;
layout( location = 4 ) uniform mat4 projMat;
layout( location = 8 ) uniform float maxLifeTime;
layout( location = 10 )uniform sampler3D fireTex;
layout( location = 11 ) uniform sampler2D framebufferDepthTexture;
layout( location = 12 ) uniform vec2 screenResolution;
layout( location = 13 ) uniform float frontClipPlaneDistance;
layout( location = 14 ) uniform mat4 invProjMatrix;

layout( points ) in;
layout( triangle_strip, max_vertices = 4 ) out;

flat in  float gLifetime[];
flat in  float gType[];
flat in  vec2  gScale[];


out vec2  texCoord;
flat out float fLifetime;
flat out float fType;

flat out float  particleRadius;
flat out vec4   particleCenterPosition; //particle center in camera space
out		 vec4   billboardPoint; //in cam space



void main()
{
	//->eye space
	vec4 pos = viewMat * gl_in[0].gl_Position;
	fLifetime = gLifetime[0];
	fType = gType[0];

	particleCenterPosition = pos;
	
	//float bw = QUADWIDTH * gScale[0].x; 
	//float bh = QUADHEIGHT * gScale[0].x;

	float bw = QUADWIDTH * (smoothstep( 0.0, maxLifeTime * 0.5, gLifetime[0] )) * (1.0 - smoothstep(  maxLifeTime * 0.5, maxLifeTime, gLifetime[0] ));
	float bh = bw;

	particleRadius = bw;
	
	
	//bottom left:
	gl_Position = pos;
	gl_Position.x -= bw;
	gl_Position.y -= bh;
	billboardPoint = gl_Position;
	gl_Position = projMat * gl_Position;
	texCoord = vec2( 0, 0 );
	EmitVertex();
	
	//top left:
	gl_Position = pos;
	gl_Position.x -= bw;
	gl_Position.y += bh;
	billboardPoint = gl_Position;
	gl_Position = projMat * gl_Position;
	texCoord = vec2( 0, 1 );
	EmitVertex();
	
	//bottom right:
	gl_Position = pos;
	gl_Position.x += bw;
	gl_Position.y -= bh;
	billboardPoint = gl_Position;
	gl_Position = projMat * gl_Position;
	texCoord = vec2( 1, 0 );
	EmitVertex();
	
	//top right:
	gl_Position = pos;
	gl_Position.x += bw;
	gl_Position.y += bh;
	billboardPoint = gl_Position;
	gl_Position = projMat * gl_Position;
	texCoord = vec2( 1, 1 );
	EmitVertex();
	
	EndPrimitive();
}