#version 330
#extension GL_ARB_explicit_uniform_location : enable

layout( location = 0 ) uniform mat4	projectionMatrix; //eye space -> homogenous clip space (-> perspective division -> device coords)
layout( location = 4 ) uniform mat4	viewMatrix; //world space -> eye space
layout( location = 7 ) uniform mat4	modelMatrix;

layout( location = 11) uniform bool	instancingUsed; //-> used in mat4 modelMatrixI instead of the uniform modelMatrix!
in		mat4	modelMatrixInstance; 

layout( location = 12) uniform bool ortogonalProjection; //is true, then just use depth, if false then it is a perspective projection, use length instead
layout( location = 13) uniform float zFarPlane;

uniform float zNearPlane;


in vec4 vertex; //vertexposition



//outs are then in eye space
out float fragmentEyeDepth; //depth in eye space



float linstep(float low, float high, float v){
    return clamp( (v - low)/(high - low), 0.0, 1.0);
}



/*float normalizeDepth( float depth ){
	return linstep( -zNearPlane, -zFarPlane, depth );
}*/

float normalizeDepth( float depth ){
	return linstep( zNearPlane, zFarPlane, depth );
}


void main(void)
{
	mat4 modelMatrixUsed = modelMatrix;
	if( instancingUsed == true ) modelMatrixUsed = modelMatrixInstance;
	
	mat4 modelViewMatrix = viewMatrix * modelMatrixUsed;

	vec4 fragmentEye = modelViewMatrix * vertex; //just the opposition of the view ray, but not the projection perspective needed
	
	if( ortogonalProjection == true ) fragmentEyeDepth = normalizeDepth( -fragmentEye.z ); 
	else fragmentEyeDepth = normalizeDepth( length( fragmentEye ) ); 

	gl_Position = projectionMatrix *  fragmentEye; //vertex transformation, right transformation happens first

}
