#version 330
#extension GL_ARB_explicit_uniform_location : enable

#define LENSFLARE_WIDTH 0.1


layout( location = 0 ) uniform mat4	projectionMatrix; //eye space -> homogenous clip space (-> perspective division -> device coords)

layout( location = 11 ) uniform vec3 sunVector;
layout( location = 21 ) uniform sampler2D framebufferDepthTexture;

layout( triangles ) in;

//change the max vertices output -> 4 + numberOfFlares * 4
layout( triangle_strip, max_vertices = 16 ) out; 

out vec2 texCoords;
flat out float cosImagePlane;




void main()
{
	//test if there is an occluder between the flares and the sun
	vec4 sunProj = projectionMatrix * vec4(sunVector, 1.0);
	sunProj /= sunProj.w;
	sunProj.xyz = sunProj.xyz * 0.5 + 0.5;
	float texDepth = texture( framebufferDepthTexture, sunProj.xy );

	if( texDepth < sunProj.z ) return;


	vec2 leftLow = vec2(  -LENSFLARE_WIDTH, -LENSFLARE_WIDTH );
	vec2 leftHigh = vec2( -LENSFLARE_WIDTH,  LENSFLARE_WIDTH );
	vec2 rightLow = vec2(  LENSFLARE_WIDTH, -LENSFLARE_WIDTH );
	vec2 rightHigh = vec2( LENSFLARE_WIDTH,  LENSFLARE_WIDTH );

	vec4 sunEye = vec4(sunVector, 1.0);

	vec4 reflectionVector = vec4( 0.0, 0.0, -5.0, 1.0 );

	vec4 flareDir = normalize( reflectionVector - sunEye );


	cosImagePlane = max( dot( -flareDir.xyz, vec3( 0.0, 0.0, -1.0 ) ), 0.0 );

	cosImagePlane *= 1.0 - smoothstep( 0.85, 1.0, cosImagePlane );

	if( cosImagePlane <= 0.0 ) return; //angle too flat, no flares



	for( int i = 0; i < 4; ++i )
	{
		vec4 pos =  reflectionVector + flareDir * (i+1);

		gl_Position = pos;

		gl_Position.xy += leftLow;
		texCoords = vec2( 0.25*i, 0.0 );
		gl_Position = projectionMatrix * gl_Position;
		EmitVertex();


		gl_Position = pos;

		gl_Position.xy += rightLow;
		texCoords = vec2( 0.25*(i+1), 0.0 );
		gl_Position = projectionMatrix * gl_Position;
		EmitVertex();

		gl_Position = pos;

		gl_Position.xy += leftHigh;
		texCoords = vec2( 0.25*i, 1.0 );
		gl_Position = projectionMatrix * gl_Position;
		EmitVertex();

		gl_Position = pos;

		gl_Position.xy += rightHigh;
		texCoords = vec2( 0.25*(i+1), 1.0 );
		gl_Position = projectionMatrix * gl_Position;
		EmitVertex();


		EndPrimitive();
	}
}