#version 330
#extension GL_ARB_explicit_uniform_location : enable


layout( location = 21 ) uniform sampler2D downsampledScreen;
layout( location = 22 ) uniform sampler1D lensColor;

layout( location = 10 ) uniform float ghostDispersal; 
layout( location = 11 ) uniform int   ghostNumber;
layout( location = 12 ) uniform float haloWidth;
layout( location = 13 ) uniform float distortion;


noperspective in vec2 		fragmentTexCoords;


layout(location = 0 ) out vec4 fragColor;


//direction -> direction of distortion
//distortionv -> per channel distortion factor
vec4 textureDistorted( in sampler2D tex, in vec2 texCoord, in vec2 direction, in vec3 distortionv )
{
	return vec4( texture( tex, texCoord + direction * distortionv.r ).r, 
				 texture( tex, texCoord + direction * distortionv.g ).g, 
				 texture( tex, texCoord + direction * distortionv.b ).ba );
}



void main(void)
{	
	//flip coordinates because the lens reflection are on the "other side"
	vec2 fragCoord = -fragmentTexCoords + vec2( 1.0 );

		//ghost vector to the image center
	vec2 ghostVec = (vec2( 0.5 ) - fragCoord ) * ghostDispersal;



	vec2 texelSize = 1.0 / vec2( textureSize( downsampledScreen, 0 ) );
	vec3 distortionv = vec3( -texelSize.x * distortion, 0.0, texelSize.x * distortion );
	vec2 direction = normalize( ghostVec );



	//sample ghosts:
	fragColor = vec4( 0.0 );

	for( int i = 0; i < ghostNumber; ++i ){

		//NOT HALO:
		vec2 offset = fract( fragCoord + ghostVec * float(i) );

		//to allow only bright spot we weight the samples by a falloff to outside
		float weight = length( vec2( 0.5 ) - offset ) / length( vec2( 0.5 ) );
		weight = pow( 1.0 - weight, 10.0 );
		//its a linear falloff, bright spots in the center of the input image can cast ghosts to the edges, but bright spots at the edges cant cast ghosts to the center

		//fragColor += texture( downsampledScreen, offset ) * weight;
		fragColor += textureDistorted( downsampledScreen, offset, direction, distortionv ) * weight;


		//HALO:
		vec2 haloVec = normalize( ghostVec ) * haloWidth;
		weight = length( vec2( 0.5 ) - fract( fragCoord + haloVec ) ) / length( vec2( 0.5 ) );
		weight = pow( 1.0 - weight, 5.0 ); //change the pow coeff to make the ring bigger/smaller
		//fragColor += texture( downsampledScreen, fragCoord + haloVec ) * weight;
		fragColor += textureDistorted( downsampledScreen, fragCoord + haloVec, direction, distortionv ) * weight;
	}

	//fragColor *= texture( lensColor, length( vec2( 0.5 ) - fragCoord ) / length( vec2( 0.5 ) ) );


	fragColor.a = 1.0;
	
}