#version 330
#extension GL_ARB_explicit_uniform_location : enable


layout( location = 20 ) uniform sampler2D framebufferObjectTexture; //the framebuffer rendered in a texture
layout( location = 21 ) uniform sampler2D framebufferDepthTexture;

layout( location = 0 ) uniform mat4 invProjMatrix;

//layout( location = 4 ) uniform mat4 invCurrViewMat; 
//layout( location = 7 ) uniform mat4 previousProjViewMat; ->optimization
layout( location = 4 ) uniform mat4 prevToCurrentProjView;

layout( location = 11) uniform float currentFPS;

#define MOTION_BLUR_SAMPLES 10
#define TARGET_FPS 60


noperspective in vec2 	fragmentTexCoords;

layout(location = 0 ) out vec4 fragColor;





vec4 getPixelPosViewSpace() //actually just unprojecting
{
	float z = texture( framebufferDepthTexture, fragmentTexCoords );
	z = z * 2.0 - 1.0;
	float x = fragmentTexCoords.x * 2.0 - 1.0;
	float y = fragmentTexCoords.y * 2.0 - 1.0;
	vec4 projectedPos = vec4( x, y, z, 1.0 );
	projectedPos = invProjMatrix * projectedPos;
	projectedPos /= projectedPos.w; 
	return projectedPos;
}



void main(void)
{	
	//pixels current view space positon
	vec4 pixelViewSpace = getPixelPosViewSpace();

	//move it to world space:
	///vec4 pixelWorldSpace = invCurrViewMat * pixelViewSpace;
	//now its in world space... so lets put it in the previous view space

	////vec4 prevViewSpace = previousProjViewMat * pixelWorldSpace;
	//now this pixel is in the previous frames screen space

	//homogenization:
	//prevViewSpace /= prevViewSpace.w;

	//!Optimization:
	vec4 prevViewSpace = prevToCurrentProjView * pixelViewSpace;
	prevViewSpace /= prevViewSpace.w;
	prevViewSpace.xy = prevViewSpace.xy * 0.5 + 0.5;
	//!

	//put to tex coords:
	vec2 texCoordsLastFrame = prevViewSpace.xy;

	//now i have the offset vector
	vec2 offset = texCoordsLastFrame - fragmentTexCoords;

	//normalize is with the framerate
	offset *= currentFPS / float( TARGET_FPS );

	fragColor = texture( framebufferObjectTexture, fragmentTexCoords );

	for( int i = 1; i < MOTION_BLUR_SAMPLES; ++i ){
		vec2 texoffset = offset * (float(i) / float( MOTION_BLUR_SAMPLES - 1) - 0.75);
		fragColor.rgb += texture( framebufferObjectTexture, fragmentTexCoords + texoffset ).rgb;
	}

	fragColor.rgb /= float(MOTION_BLUR_SAMPLES);

	fragColor.a = clamp( fragColor.a, 0.0, 1.0 );

}

