#version 330
#extension GL_ARB_explicit_uniform_location : enable



layout( location = 20 ) uniform sampler2D screenTex; //the framebuffer rendered in a texture
//layout( location = 21 ) uniform sampler2D luminanceTex;


layout( location = 5 ) uniform int   mipLevel;
layout( location = 6 ) uniform float manualExposure;
layout( location = 7 ) uniform float keyValue; //the middle grey value, the higher it gets the lighter is the scene
layout( location = 8 ) uniform int	 toneMapper;
layout( location = 9 ) uniform float saturation;
layout( location = 10 ) uniform float maxWhite;


noperspective in vec2 		fragmentTexCoords;


layout(location = 0 ) out vec4 fragColor;


const vec3 lumconvert = vec3( 0.2126, 0.7152, 0.0722 );





vec3 unknownToneMap( vec3 pixelColor, float exposure )
{
	pixelColor = ( pixelColor * (1.0 + ( pixelColor / exposure * exposure) ) ) / ( 1.0 + pixelColor );
	pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}


//http://filmicgames.com/archives/75
vec3 linearToneMap( vec3 pixelColor, float pixelLum, float exposure )
{
	pixelLum = pixelLum * exposure;
	pixelColor *= pixelLum  * saturation;
	pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}



vec3 reinhardToneMap( vec3 pixelColor, float pixelLum, float exposure)
{
	pixelLum = pixelLum * exposure;
	pixelLum = pixelLum * (1.0 + pixelLum / (maxWhite * maxWhite)) / (1.0 + pixelLum );
	pixelColor *= pixelLum  * saturation;
	//pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}


vec3 reinhardToneMap2( vec3 pixelColor, float pixelLum, float exposure)
{
	pixelLum = pixelLum * exposure * saturation;
	pixelColor *= pixelLum;
	pixelColor = pixelColor * (1.0 + pixelColor / (maxWhite * maxWhite)) / (1.0 + pixelColor );
	pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}





vec3 hejlToneMap( vec3 pixelColor, float pixelLum, float exposure )
{
	pixelLum = pixelLum * exposure;
	pixelLum -= 0.004;
	pixelLum = max( pixelLum, 0.0 );
	pixelColor *= pixelLum  * saturation;
	pixelColor = (pixelColor * (6.2 * pixelColor + 0.5 ) )/( pixelColor * (6.2 * pixelColor + 1.7 ) + 0.06 );
	return pixelColor;
}



float A = 0.15;
float B = 0.50;
float C = 0.10;
float D = 0.20;
float E = 0.02;
float F = 0.30;
float W = 11.2;

vec3 unchartedToneMap( vec3 pixelColor, float pixelLum, float exposure )
{
	pixelLum = pixelLum * exposure;
	float exposureBias = 2.0;
	pixelLum = exposureBias * pixelLum;
	pixelLum = ((pixelLum * (A * pixelLum + C * B) + D * E) / ( pixelLum * (A * pixelLum + B) + D * F ) ) - E / F;

	float whiteScale = 1.0 / ((maxWhite * (A * maxWhite + C * B) + D * E) / (maxWhite * (A * maxWhite + B) + D * F ) ) - E / F;
	pixelLum *= whiteScale;

	pixelColor *= pixelLum  * saturation;;
	pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}



vec3 unchartedToneMap2( vec3 pixelColor, float pixelLum, float exposure )
{
	pixelLum = pixelLum * exposure;
	float exposureBias = 2.0;
	pixelLum = exposureBias * pixelLum;
	pixelColor *= pixelLum;
	pixelColor = ((pixelColor * (A * pixelColor + C * B) + D * E) / ( pixelColor * (A * pixelColor + B) + D * F ) ) - E / F;

	float whiteScale = 1.0 / ((W * (A * W + C * B) + D * E) / ( W * (A * W + B) + D * F ) ) - E / F;
	pixelColor *= whiteScale * saturation;

	pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}


vec3 exponentialToneMap( vec3 pixelColor, float pixelLum, float exposure )
{
	pixelColor *= pixelLum;
	pixelColor = 1.0 - exp(-exposure * pixelColor);
	//pixelColor = vec3( pow( pixelColor.r, 1.0 / 2.2 ), pow( pixelColor.g, 1.0 / 2.2 ), pow( pixelColor.b, 1.0 / 2.2 ) );
	return pixelColor;
}






void main(void)
{	
	//float lumAvg = textureLod( luminanceTex, fragmentTexCoords, mipLevel ).r;
	//lumAvg = exp( lumAvg ); //->full average log luminance... approximation of the key of the scene

	float lumAvg = dot( textureLod( screenTex, fragmentTexCoords, mipLevel ).rgb, lumconvert );

	vec4 pixelColorTex = texture( screenTex, fragmentTexCoords );
	vec3 pixelColor = pixelColorTex.rgb;
	float lumPixel = dot( pixelColor, lumconvert );
	lumPixel += 0.00001;

	//Scaling of pixel luminance comapared to average scene luminance
	float lumScaled = keyValue / lumAvg * lumPixel;

	float exposure = manualExposure;
	//exposure = keyValue / lumAvg; //->Automatic

	vec3 tonemapped = vec3( 0.0, 0.0, 0.0 );
	if( toneMapper == 0 ) tonemapped = reinhardToneMap2( pixelColor, lumScaled, exposure );
	else if( toneMapper == 1 ) tonemapped = reinhardToneMap( pixelColor, lumScaled, exposure  );
	else if( toneMapper == 2 ) tonemapped = hejlToneMap( pixelColor, lumScaled, exposure  );
	else if( toneMapper == 3 ) tonemapped = unchartedToneMap( pixelColor, lumScaled, exposure  );
	else if( toneMapper == 4 ) tonemapped = exponentialToneMap( pixelColor, lumScaled, exposure  );

	fragColor = vec4( tonemapped, 1.0 );
}


